/// <reference path="./includes/jquery.d.ts" />
/// <reference path="./includes/require.d.ts" />
define(["require", "exports"], function(require, exports) {
    /*
    Board Game Engine
    */
    /* Possible Difficulty Settings */
    (function (Difficulty) {
        Difficulty[Difficulty["Easy"] = 0] = "Easy";
        Difficulty[Difficulty["Medium"] = 1] = "Medium";
        Difficulty[Difficulty["Hard"] = 2] = "Hard";
    })(exports.Difficulty || (exports.Difficulty = {}));
    var Difficulty = exports.Difficulty;

    (function (Events) {
        var PlayerMove = (function () {
            function PlayerMove(player, from, to) {
                this.player = player;
                this.from = from;
                this.to = to;
            }
            return PlayerMove;
        })();
        Events.PlayerMove = PlayerMove;
    })(exports.Events || (exports.Events = {}));
    var Events = exports.Events;

    /* Player Class */
    var Player = (function () {
        function Player(name, board) {
            this.Ready = false;
            this.IsComputer = false;
            this.Difficulty = Difficulty.Easy;
            this.Name = name;
            this.Board = board;
        }
        Player.prototype.Place = function (piece, place) {
            piece.Player = this;
            piece.Place(this.Board, place);
            this.fire("player:placed", new Events.PlayerMove(this, null, place));
        };
        Player.prototype.Move = function (piece, place) {
            if (piece.Player != this)
                throw "This player does not own this piece.";
            piece.Move(place);
            this.fire("player:moved", new Events.PlayerMove(this, null, place));
        };
        Player.prototype.name = function (name) {
            if (name === undefined)
                return this.Name;
            this.Name = name;
            this.fire("player:namechange", this);
            return this;
        };
        Player.prototype.ready = function (ready) {
            if (ready === undefined)
                return this.Ready;
            this.Ready = ready;
            this.fire("player:ready", this);
            return this;
        };
        Player.prototype.TakeTurn = function (place) {
        };
        Player.prototype.AutoMove = function () {
        };

        Player.prototype.on = function (eventname, action) {
            this.Board.on(eventname, action);
        };
        Player.prototype.fire = function (eventname, data) {
            if (typeof data === "undefined") { data = undefined; }
            this.Board.fire(eventname, data);
        };
        return Player;
    })();
    exports.Player = Player;

    var GamePiece = (function () {
        function GamePiece(type) {
            this.Type = type;
            this.$ = $(document.createElement("div"));
        }
        GamePiece.prototype.Place = function (board, place) {
            place.Hold(this);
        };
        GamePiece.prototype.Move = function (place) {
            if (this.Board === undefined)
                throw "You cannot move a peice not on a board.";
            place.Hold(this);
        };

        GamePiece.prototype.appendTo = function ($) {
            this.$.appendTo($);
            return this;
        };
        GamePiece.prototype.prependTo = function ($) {
            this.$.prependTo($);
            return this;
        };
        GamePiece.prototype.css = function (style) {
            this.$.css(style);
            return this;
        };
        GamePiece.prototype.style = function (classname) {
            this.$.addClass(classname);
            return this;
        };
        GamePiece.prototype.remove = function () {
            this.$.remove();
            this.Board = undefined;
            return this;
        };
        GamePiece.prototype.click = function (func) {
            this.$.click(func);
            return this;
        };
        GamePiece.prototype.on = function (eventname, action) {
            this.Board.on(eventname, action);
        };
        GamePiece.prototype.fire = function (eventname, data) {
            if (typeof data === "undefined") { data = undefined; }
            this.Board.fire(eventname, data);
        };
        return GamePiece;
    })();
    exports.GamePiece = GamePiece;

    var BoardPlace = (function () {
        function BoardPlace(board) {
            this.Board = board;
            this.$ = $(document.createElement("div")).appendTo(this.Board.$);
        }
        BoardPlace.prototype.Holding = function () {
            if (this.Holds === undefined)
                return false;
            return true;
        };
        BoardPlace.prototype.Hold = function (piece) {
            if (this.Holding())
                throw "This game place is already holding a piece.";
            this.Holds = piece;
            this.Holds.appendTo(this.$);
            return this;
        };
        BoardPlace.prototype.Release = function () {
            if (this.Holding())
                this.Holds.remove();
            this.Holds = undefined;
        };

        BoardPlace.prototype.css = function (style) {
            this.$.css(style);
            return this;
        };
        BoardPlace.prototype.style = function (classname) {
            this.$.addClass(classname);
            return this;
        };
        BoardPlace.prototype.remove = function () {
            this.$.remove();
            return this;
        };
        BoardPlace.prototype.click = function (func) {
            this.$.click(func);
            return this;
        };
        BoardPlace.prototype.on = function (eventname, action) {
            this.Board.on(eventname, action);
        };
        BoardPlace.prototype.fire = function (eventname, data) {
            if (typeof data === "undefined") { data = undefined; }
            this.Board.fire(eventname, data);
        };
        return BoardPlace;
    })();
    exports.BoardPlace = BoardPlace;

    var GameBoard = (function () {
        function GameBoard(game) {
            this.Places = [];
            this.$ = $(document.createElement("div"));
            this.Game = game;
            this.AddPlaces();
        }
        GameBoard.prototype.AddPlaces = function () {
        };
        GameBoard.prototype.AddPlace = function (place) {
            this.Places.push(place);
            place.Position = this.Places.length;
            return this;
        };
        GameBoard.prototype.GetPlaceIndex = function (givenplace) {
            return givenplace.Position;
        };

        GameBoard.prototype.appendTo = function ($) {
            this.$.appendTo($);
            return this;
        };
        GameBoard.prototype.prependTo = function ($) {
            this.$.prependTo($);
            return this;
        };
        GameBoard.prototype.css = function (style) {
            this.$.css(style);
            return this;
        };
        GameBoard.prototype.style = function (classname) {
            this.$.addClass(classname);
            return this;
        };
        GameBoard.prototype.remove = function () {
            this.$.remove();
            return this;
        };
        GameBoard.prototype.click = function (func) {
            this.$.click(func);
            return this;
        };
        GameBoard.prototype.on = function (eventname, action) {
            this.Game.on(eventname, action);
        };
        GameBoard.prototype.fire = function (eventname, data) {
            if (typeof data === "undefined") { data = undefined; }
            this.Game.fire(eventname, data);
        };
        return GameBoard;
    })();
    exports.GameBoard = GameBoard;

    var Game = (function () {
        function Game(board, element) {
            this.Players = [];
            this.Started = false;
            this.$ = $(document.createElement("div")).appendTo(element);
            this.Board = board.appendTo(this.$);
        }
        Game.prototype.AddPlayer = function (player) {
            this.Players.push(player);
            this._Player = player;
            this.fire("player:added", player);
            return this;
        };
        Game.prototype.RemoveAllPlayers = function () {
            this.Players = [];
            this.fire("player:clear");
        };
        Game.prototype.CurrentPlayer = function (player) {
            if (player === undefined) {
                if (this._Player === undefined)
                    throw "No players have been added to this game.";
                return this._Player;
            }
            this._Player = player;
        };
        Game.prototype.Start = function () {
            $.each(this.Players, function (index, player) {
                if (!player.ready())
                    throw "Not all players are ready.";
            });
            this.Started = true;
            this.fire("game:started");
        };

        Game.prototype.on = function (eventname, action) {
            this.$.on(eventname, action);
        };
        Game.prototype.fire = function (eventname, data) {
            if (typeof data === "undefined") { data = undefined; }
            this.$.trigger(eventname, data);
        };

        /* Makes the current player take their turn */
        Game.prototype.TakeTurn = function (place) {
            if (this.Started == false)
                this.Start();
            this.CurrentPlayer().TakeTurn(place);

            var player = this.CheckWin();
            if (!player === undefined)
                this.Win(player);
        };

        /*
        Moves the player onto the next one (for example after taking a turn
        Default behaviour is to move on to the next player in the list - or the first if at the end of the list.
        */
        Game.prototype.CyclePlayers = function () {
            var playernumber = 0;
            for (var i = 0; i < this.Players.length; i++) {
                if (this.Players[i] === this._Player) {
                    playernumber = i;
                    break;
                }
            }
            playernumber = playernumber + 1;
            if (!this.Players[playernumber])
                this._Player = this.Players[0];
else
                this._Player = this.Players[playernumber];

            this.fire("player:yourturn", this._Player);
        };

        Game.prototype.CheckWin = function () {
            return undefined;
        };
        Game.prototype.Win = function (player) {
            console.log(["win", player]);
            this.fire("game:win", player);
        };
        Game.prototype.Draw = function () {
            this.fire("game:draw");
        };
        return Game;
    })();
    exports.Game = Game;
});
