/// <reference path="../includes/jquery.d.ts" />
/// <reference path="../includes/require.d.ts" />

import BoardGame = require('boardgame');

/* Tic Tac Toe */
class TicTacToe extends BoardGame.Game {
	constructor(element: JQuery) { 
		super(new TicTacToeBoard(this), element);

		/* Style the board */
		this.$.addClass("TicTacToeBoard");

		this.on("player:ready", ()=> {
			console.log("player is ready");
		});

		/* Cycle player turns when players take a turn */
		this.on("player:placed", ()=> {
			this.CyclePlayers();
		});

		this.on("player:placed", (event: any, move: BoardGame.Events.PlayerMove)=> {
			
			var winner: BoardGame.Player = this.WhoWins(move);
			if (winner!==undefined) this.fire("player:win", winner);
		});

		/* Ensure correct board size */
		$(window).on('resize', () => { this.OnResize(); });
		this.OnResize();
	}

	public OnResize() {
		this.$.width(this.$.height());
	}

	public WhoWins(move: BoardGame.Events.PlayerMove) : BoardGame.Player {
		var position: number = this.Board.GetPlaceIndex(move.to) - 1;
		var playerspiece: BoardGame.GamePiece = move.to.Holds;

		/* Vertical Check */
		if (this.CheckPeiceAt(playerspiece, position - 6) + this.CheckPeiceAt(playerspiece, position - 3) + this.CheckPeiceAt(playerspiece, position) + this.CheckPeiceAt(playerspiece, position + 3) + this.CheckPeiceAt(playerspiece, position + 6) >=3 )
			return move.player;


		/* Horizontal Check */
		if (this.CheckPeiceAt(playerspiece, position - 2) + this.CheckPeiceAt(playerspiece, position - 1) + this.CheckPeiceAt(playerspiece, position) + this.CheckPeiceAt(playerspiece, position + 1) + this.CheckPeiceAt(playerspiece, position + 2) >=3 )
			return move.player;

		/* Left Diagonal Check */
		if (this.CheckPeiceAt(playerspiece, 0) + this.CheckPeiceAt(playerspiece, 4) + this.CheckPeiceAt(playerspiece, 8) == 3) return move.player;

		/* Right Diagonal Check */
		if (this.CheckPeiceAt(playerspiece, 2) + this.CheckPeiceAt(playerspiece, 4) + this.CheckPeiceAt(playerspiece, 6) == 3) return move.player;

		return undefined;
	}

	private CheckPeiceAt(piece : BoardGame.GamePiece, position: number): number {
		var check: BoardGame.BoardPlace = this.Board.Places[position];
		if (check === undefined || check.Holds === undefined) return 0;

		if (check.Holds.Type == piece.Type) return 1;
		return 0;
	}
}	

class TicTacToeBoard extends BoardGame.GameBoard {
	constructor(game: BoardGame.Game) { super(game);
		this.style("TicTacToeBoard");
	}
	public AddPlaces() {
		/* Create 9 board squares */
		for (var i : number = 0; i < 9; i++) {
			this.AddPlace(new TicTacToePlace(this));
		}
	}
}

class TicTacToePlace extends BoardGame.BoardPlace {
	constructor(board: TicTacToeBoard) { super(board);
		this.style("TicTacToePlace")
			.click(() => {
				this.Board.Game.TakeTurn(this);
			});
	}
}

export class TicTacToePlayer extends BoardGame.Player {
	public PlayerType : string;

	constructor(name: string, board:BoardGame.GameBoard) { super("", board);
		this.ready(true);
		this.PlayerType = name;
	}

	public TakeTurn(place : BoardGame.BoardPlace) {
		this.Place((<BoardGame.GamePiece>new GamePieces[this.PlayerType]()), place);	
	}

	public IconSrc() {
		if (this.PlayerType=="Naught") 		return "../img/o.png";
		else if (this.PlayerType=="Cross") 	return "../img/x.png";
	}

	public Color() {
		if (this.PlayerType=="Naught") 		return "rgb(185,74,72)";
		else if (this.PlayerType=="Cross") 	return "rgb(58,135,173)";
	}
}

module GamePieces {
	export class Naught extends BoardGame.GamePiece {
		constructor() { super("Naught");
			this.$.append(
				$(document.createElement("img")).attr("src", "../img/o.png").css({ "width": "100%", "height": "100%", "padding": "10px"})
			)
			this.$.css({
				"width": "100%",
				"height": "100%",
				"margin": "0px !important",
				"padding" : "10% !important",
				"background-color" : "rgb(185,74,72)"
			});
		}
	}

	export class Cross extends BoardGame.GamePiece {
		constructor() { super("Cross");
			this.$.append(
				$(document.createElement("img")).attr("src", "../img/x.png").css({ "width": "100%", "height": "100%", "padding": "10px"})
			)
			this.$.css({
				"width": "100%",
				"height": "100%",
				"margin": "0px !important",
				"padding" : "10% !important",
				"background-color" : "rgb(58,135,173)"
			});
		}
	}
}



var ttt: TicTacToe;
export module Controller {

	export function NewGame() {
		$("#NewGamePanel").show();
		$("#GameBoard").hide();
		$("#AboutPanel").hide();
		// if (ttt.started==true) 
	}

	export function ShowAbout() {
		$("#NewGamePanel").hide();
		$("#GameBoard").hide();
		$("#AboutPanel").show();
	}
	export function ShowGame() {
		$("#NewGamePanel").hide();
		$("#GameBoard").show();
		
		$("#AboutPanel").hide();
	}
	export function CreateNewGame() {
		ShowGame();
		$("#GameBoard").empty();

		ttt = undefined;
		ttt = ttt = new TicTacToe($("#GameBoard"));

		var Naught : TicTacToePlayer = new TicTacToePlayer("Naught", ttt.Board);
			Naught.Name = $("#Player1Name").val()!="" ? $("#Player1Name").val() : "Player 1";
		var Cross: TicTacToePlayer = new TicTacToePlayer("Cross", ttt.Board);
			Cross.Name = $("#Player2Name").val()!="" ? $("#Player2Name").val() : "Player 2";
		ttt.AddPlayer(Naught);
		ttt.AddPlayer(Cross);

		$("#NavResumeGame small").text(ttt.Players[0].Name + " vs " + ttt.Players[1].Name);
		$("#DetailsText").hide();
		$("#NavResumeGame").show();
		$(".nav button.active").removeClass("active");
		$("#NavResumeGame").addClass("active");
		

		

		ttt.on("player:yourturn", TurnChanged);
		ttt.on("player:win", PlayerWin);
		$('#WinModal').on('hide.bs.modal', () => {
			$("#TurnInfo").hide();
			$("#NavResumeGame").hide();
			$("#NavNewGame").addClass("active");
			NewGame();
		})
		TurnChanged(undefined, ttt.CurrentPlayer());
	}

	/* What to do when the player whos turn it is has changed */
	export function TurnChanged(event: any, player: TicTacToePlayer) {
		$("#TurnInfo").show();
		$("#TurnInfo #TurnPlayerText").text(player.Name+"'s turn.");
		$("#TurnInfo #TurnPlayerImage")
			.attr("src", player.IconSrc())
			.css("background-color", player.Color());
	}

	export function PlayerWin(event: any, player: TicTacToePlayer) {
		ttt.Started = false;

		(<any>$('#WinModal')).modal("show");

		$('#WinText').empty();

		$('#WinText')
			.append($(document.createElement("span")).text(player.Name+" has won."));
	}
}


$(() => {
	/* Show initial start field */
	Controller.NewGame();
	$("#TurnInfo").hide();

	/* Navigation Pressed */
	$(".nav button").click(()=> {
		$(".nav button.active").removeClass("active");
	});

	/* Resume Button */
	$("#NavResumeGame").hide();
	$("#NavResumeGame").click(()=> {
		Controller.ShowGame();
		$("#NavResumeGame").addClass("active");
	});

	$("#NavNewGame").click(()=> {
		$("#NavNewGame").addClass("active");
		Controller.NewGame();
	});

	$("#StartGameButton").click(()=> {
		Controller.CreateNewGame();
	});

	$("#NavAbout").click(()=> {
		$("#NavAbout").addClass("active");
		Controller.ShowAbout();
	});
});